package org.hmn.jrsd;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.io.Reader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.MissingResourceException;

import org.antlr.stringtemplate.NoIndentWriter;
import org.antlr.stringtemplate.StringTemplate;
import org.antlr.stringtemplate.StringTemplateGroup;
import org.antlr.stringtemplate.language.AngleBracketTemplateLexer;
import org.antlr.stringtemplate.language.DefaultTemplateLexer;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.TTCCLayout;

import antlr.RecognitionException;
import antlr.TokenStreamException;

/*
 * 
 * Cree par Harry Karadimas a 29 mars 2006
 */
/**
 * The Jrsd application; called either via main() or via {@link JrsdTask}.
 * @author karadimas
 */
public class Jrsd
{
  public static final int SECOND = 1000;
  public static final int MINUTE = SECOND * 60;
  public static final int HOUR = MINUTE * 60;
  public static final int DAY = HOUR * 24;
  
  static Logger lg = Logger.getLogger(Jrsd.class);
  public static boolean forceFieldsToUppercase = false;
  public static boolean forceFieldsToLowercase = false;
  
  boolean help = false;
  InputStream in;
  String srcRootPath;
  String sqlRootPath;
  String xmlRootPath = ".";
  File srcRoot;
  File sqlRoot;
  String logPath;
  boolean overwrite = false;
  boolean debug = false;
  boolean stdin = true;
  boolean stdout = true;
  public String NL = System.getProperty("line.separator");
  ArrayList<String> inputs = new ArrayList<String>();
  String tempPrefix;
  String fileext = "sql";
  //use "-genoracomments" attribute/option to set to true
  boolean generateComments = false;   
  boolean noJava = false;
  boolean noSql = false;
  String encoding = Charset.defaultCharset().name();
  boolean emitXml = false;
  
  /**
   * Look if name should better be quoted.
   * @param name
   * @return True if name does not start with a letter, or contains something
   *   else than letters, digits, underscores.
   */
  public static boolean isQuotingNecessary(String name)
  {
    char[] ca = name.toCharArray();
    for (int i = 0; i < ca.length; i++) {
      char c = Character.toUpperCase(ca[i]);
      if ('A' <= c & c <= 'Z') continue;      
      if (i > 0 & ('0' <= c & c <= '9')) continue;
      if (i > 0 & c == '_') continue;
      return true;
    }//for
    return false;
  }
  
  public static String condFieldCase(String name)
  {
    if (name == null) return null;
    if (forceFieldsToUppercase) return name.toUpperCase();
    if (forceFieldsToLowercase) return name.toLowerCase();
    return name;
  }
  
  public Jrsd(String[] args)
  throws JrsdException
  {
    //read and interpret command-line options
    for (int i = 0; i < args.length; i++) {
      String arg = args[i].intern(); //pour pouvoir utiliser '=='
      if (arg.equals("-srcroot")) {        
        if (++i >= args.length) throw new RuntimeException("Missing argument for -srcroot");
        srcRootPath = args[i];        
      }
      else if (arg.equals("-sqlroot")) {        
        if (++i >= args.length) throw new RuntimeException("Missing argument for -sqlroot");
        sqlRootPath = args[i];        
      }
      else if (arg.equals("-xmlroot")) {        
        if (++i >= args.length) throw new RuntimeException("Missing argument for -xmlroot");
        xmlRootPath = args[i];        
      }
      else if (arg.equals("-log")) {
        if (++i >= args.length) throw new RuntimeException("Missing argument for -log");
        logPath = args[i];
      }
      else if (arg.equals("-fileext")) {
        if (++i >= args.length) throw new RuntimeException("Missing argument for -fileext");
        fileext = args[i];
      }
      else if (arg.equals("-encoding")) {
        if (++i >= args.length) throw new RuntimeException("missing argument for -encoding");
        encoding = args[i];
      }
      else if (arg.equals("-overwrite")) {
        overwrite = true;
      }
      else if (arg.equals("-forcefieldsuc")) {
        forceFieldsToUppercase = true;
      }
      else if (arg.equals("-forcefieldslc")) {
        forceFieldsToLowercase = true;
      }
      else if (arg.equals("-genoracomments")) {
        generateComments = true;
      }
      else if (arg.equals("-debug")) {
        debug = true;
      }
      else if (arg.equals("-nojava")) {
        noJava = true;
      }
      else if (arg.equals("-nosql")) {
        noSql = true;
      }
      else if (arg.equals("-emitxml")) {
        emitXml = true;
      }
      else if (arg.equals("-help")) {
        help = true;
      }
      else {
        inputs.add(arg);        
      }      
    }    
  }
  
  public Jrsd() {}
    
  public void init()
  throws JrsdException
  {
    //initialize the logs
    //for a finer initialization, use -Dlog4j.configuration
    if (logPath != null) {
      //if a log file is used to output the log, the console is used only
      //for the errors, and the log is used for INFO output (or DEBUG output
      //if debug is on)
      try {
        FileAppender lfa = new FileAppender(new TTCCLayout(), logPath);
        if (debug) lfa.setThreshold(Level.DEBUG);
        else lfa.setThreshold(Level.INFO);
        Logger.getRootLogger().addAppender(lfa);
        //ne garder la sortie console que pour les erreurs
        ConsoleAppender a1 = 
          (ConsoleAppender) Logger.getRootLogger().getAppender("A1");
        if (a1 != null & debug) a1.setThreshold(Level.ERROR);
      }
      catch (IOException ex) {
        throw new JrsdException(ex);
      }
    }
    else {
      //regular console output, with level INFO
      ConsoleAppender a1 = 
        (ConsoleAppender) Logger.getRootLogger().getAppender("A1");
      //the level switches to DEBUG in debug mode
      if (a1 != null & debug) a1.setThreshold(Level.DEBUG);
    }
    
    //verify that mandatory options are present
    if (noJava == false) {
      srcRoot = verifyDirectory(srcRootPath, "Source root", "-srcroot");
    }
    if (noSql == false) {
      sqlRoot = verifyDirectory(sqlRootPath, "Sql root", "-sqlroot");
    }
  }
  
  private static File verifyDirectory(String path, String desc, String opt)
  throws JrsdException
  {
    if (isEmpty(path)) {
      throw new JrsdException(desc+" path ("+opt+" <path>) missing");
    }
    File dir = new File(path);
    if (!dir.exists()) {
      throw new JrsdException(desc+" does not exist");
    }
    if (!dir.isDirectory()) {
      throw new JrsdException(desc+" is not a directory");
    }
    return dir;
  }

  private void optionError(String message) 
  throws JrsdException
  {
    usage(System.err);
    throw new JrsdException(message);
  }
  
  /**
   * Display some usage help
   * @param out
   */
  public void usage(PrintStream out) 
  {
    out.println(
    "Usage :\n" +
    "Jrsd [options] file [file ...]\n" +
    "  precompiles jrsd files to sql scripts and java source files.\n" +
    "Options :\n" +
    "-srcroot <dir> : base dir for the generated source files. Mandatory.\n" +
    "-sqlroot <dir> : base dir for the generated sql files. Mandatory.\n" +
    "-xmlroot <dir> : base dir for the generated xml files. Optional, defaults " +
    "                 to current dir.\n" +
    "-log <file> : path of the log file\n" +
    "-overwrite : overwrite output file if it exists (default is to exit with\n" +
    "  an error if the output file already exists)\n" +
    "-forcefieldsuc : force field names to uppercase\n" +
    "-forcefieldslc : force field names to lowercase\n" +
    "-genoracomments : generate comment DDL, oracle-style\n" +
    "-nosql : disable sql generation\n" +
    "-nojava : disable java code generation\n" +
    "-debug : turn on debugging mode (lots of information)\n" +
    "-help : print help and exit\n" +
    "\n" +
    "");
  }
  
  public void run()
  throws JrsdException
  {
    //si -help demand, simplement afficher l'aide et sortir.
    if (help) {
      usage(System.out);
      return;
    }
    //execute the application
    lg.debug("Starting Jrsd.");    
    String inputPath = null;
    try {
      for (Iterator<String> iter = inputs.iterator(); iter.hasNext();) {
        inputPath = iter.next();
        lg.debug("Processing input '"+inputPath+"'");
        processPath(inputPath);                
      }        
    }
    catch (JrsdException ex) {
      lg.error("JrsdException while processing file " + inputPath);
      throw ex;
    }
    catch (Exception ex) {
      lg.error("Exception while processing file " + inputPath);
      throw new JrsdException(ex);
    } 
  }
    
  private void processPath(String inputPath)
  throws FileNotFoundException, RecognitionException, TokenStreamException, IOException, JrsdException
  {
    JrsdParser parser;
    JrsdLexer lexer;
    File inputFile = new File(inputPath);
    FileReader in = new FileReader(inputFile);        
    lexer = new JrsdLexer(in);
    lexer.setFilename(inputPath);    
    parser = new JrsdParser(lexer);
    parser.setFilename(inputPath);
    lg.debug("Parsing file : "+inputPath);
    JrsdSpec spec = parser.jrsdspec();
    spec.setSpecFile(inputPath);
    spec.setSpecDate(new Date(inputFile.lastModified()));
    lg.debug("File parsed.");
    if (spec.getErrorCount() > 0) {
      lg.info("Some errors were found in file " + inputPath);
      for (Iterator<JrsdError> iter = spec.errors.iterator(); iter.hasNext();) {
        JrsdError err = iter.next();
        System.err.println(err.getLine() + " : "+err.getMessage());
      }
      throw new JrsdException("Errors in jrsd file.");      
    }
    else if (!parser.errs.isEmpty()) {
      List<?> errs = parser.errs;
      for (Object err : errs) {
        lg.error("" + err);
      }
      throw new JrsdException("Errors in jrsd file.");
    }
    else {
      produceSql(spec);
      produceMappings(spec);      
      if (emitXml) {
        String nm = inputFile.getName();
        int ix = nm.lastIndexOf('.');
        if (ix >= 0) nm = nm.substring(0, ix);
        File outputFile = new File(xmlRootPath, nm + ".xml");
        emitXml(outputFile, spec);
      }
    }
  }

  public static void main(String[] args)
  {
    try {
      Jrsd app = new Jrsd(args);
      app.init();
      app.run();
    }
    catch (JrsdException ex) {
      ex.printStackTrace();
      lg.error("Exception while running Jrsd", ex);
      LogManager.shutdown();
      System.exit(1);
    }
  }
  
  /**
   * Generate the SQL in the appropriate file
   * @param spec
   * @throws MissingResourceException
   * @throws UnsupportedEncodingException
   * @throws IOException
   * @throws JrsdException
   */
  void produceSql(JrsdSpec spec)
  throws MissingResourceException, UnsupportedEncodingException, IOException, JrsdException
  {
    if (noSql) {
      lg.debug("Sql generation turned off, returning.");
      return;
    }
    InputStream inis = getClass().getResourceAsStream("/jrsdsql.st");
    if (inis == null) {
      throw new MissingResourceException("Grammar not found", "/jrsdsql.st", "jrsdsql");
    }
    Reader in = new InputStreamReader(inis, "ISO-8859-1");
    StringTemplateGroup stGroup = new StringTemplateGroup(in, DefaultTemplateLexer.class);
    //Iterate over table defs to detect sql source conflicts. Nothing will be
    //emitted if conflicts exist.
    ArrayList<File> filesToDelete = new ArrayList<File>();
    Iterator<TableDef> iter = spec.getTableDefs().iterator();
    while (iter.hasNext()) {
      TableDef td = iter.next();
      detectSqlSourceConflicts(td, filesToDelete);
    }
    if (!filesToDelete.isEmpty()) {
      lg.info("Deleting conflicting SQL source files");
      for (Iterator<File> fiter = filesToDelete.iterator(); fiter.hasNext();) {
        File ftd = fiter.next();
        boolean deleteOk = ftd.delete();
        if (!deleteOk) throw new IOException("Can not delete file '"+ftd+"'");
      }
    }
    iter = spec.getTableDefs().iterator();
    while (iter.hasNext()) {
      TableDef td = (TableDef) iter.next();
      StringTemplate st = stGroup.getInstanceOf("tabledef");
      st.setAttribute("d", td);
      st.setAttribute("generatecomments", new Boolean(generateComments));      
      StringWriter sw = new StringWriter();
      st.write(new NoIndentWriter(sw));
      String sqlSource = sw.toString();
      writeSqlSource(td, sqlSource);
      lg.debug("sql : "+sqlSource);
    }    
  }
  
  /**
   * Generate the source code for the java classes 
   * @param spec
   * @throws IOException
   */
  void produceMappings(JrsdSpec spec)
  throws IOException
  {
    InputStream inis = getClass().getResourceAsStream("/jrsdsorm.st");
    if (inis == null) {
      throw new MissingResourceException("Template not found", "/jrsdsorm.st", "jrsdsorm");
    }
    Reader in = new InputStreamReader(inis, "ISO-8859-1");
    StringTemplateGroup stGroup = 
      new StringTemplateGroup(in,AngleBracketTemplateLexer.class);
    Iterator<MappingDecl> iter = spec.getMappingDeclList().iterator();
    while (iter.hasNext()) {
      MappingDecl md = iter.next();
      StringTemplate st = stGroup.getInstanceOf(md.getMappingType());
      st.setAttribute("m", md);      
      StringWriter sw = new StringWriter();
      st.write(new NoIndentWriter(sw));
      if (md.getMappingType().equals("simpleorm")) {
        writeJavaSource(md, sw.toString());
      }
      lg.debug(""+sw);
    }
    List<String> gloCols = spec.getGlobalColumnListAliases();
    for (Iterator<String> it2 = gloCols.iterator(); it2.hasNext();) {
      String clAlias = it2.next();
      MappingDecl mdcl = spec.getGlobalColumnList(clAlias);
      StringTemplate st = stGroup.getInstanceOf("globalset");
      st.setAttribute("m", mdcl);
      StringWriter sw = new StringWriter();
      sw.write(st.toString());
      writeJavaSource(mdcl, sw.toString());
      lg.debug(""+sw);
    }
  }
  
  /**
   * Write the java source for one mapping
   * @param md
   * @param content
   * @throws IOException
   */
  void writeJavaSource(MappingDecl md, String content)
  throws IOException
  {
    if (noJava) {
      lg.debug("Java source production disabled, returning");
      return;
    }
    Date specDate = md.getJrsdSpec().getSpecDate();
    String mn = md.getSormQualifiedClassName();
    String relPath = mn.replace('.', File.separatorChar) + ".java";
    File classFile = new File(srcRoot, relPath);
    Date classFileDate = null;
    if (classFile.exists()) classFileDate = new Date(classFile.lastModified());
    makeDirs(classFile.getParentFile());
    String packName = null;
    if (overwrite || classFileDate == null || specDate.after(classFileDate)) {
      lg.info("writing to file '"+classFile+"'");
      FileWriter out = new FileWriter(classFile);
      out.write(content);
      out.close();
      lg.debug("File written");
    }// if
    //look for a super class; if there is one, try to generate a default base class
    if (md.getSuperClass() != null) {
      String mappingClass = md.getMappingClass();
      String mcRelPath = mappingClass.replace('.', File.separatorChar) + ".java";
      File mcClassFile = new File(srcRoot, mcRelPath);
      if (!mcClassFile.exists()) {
        //generate the file
        String seq = createDefaultBaseContent(md, mcRelPath);
        FileWriter out = new FileWriter(mcClassFile);
        lg.info("writing to file '" + mcClassFile + "'");
        out.write(seq);
        out.close();
      }
    }// if
  }
  
  /**
   * Create a default base class, that the user can complete later with his
   * own methods.
   * @param md
   * @param relPath
   * @return
   * @throws IOException
   */
  String createDefaultBaseContent(MappingDecl md, String relPath)
  throws IOException
  {
    InputStream inis = getClass().getResourceAsStream("/jrsdbaseclass.st");
    if (inis == null) {
      throw new MissingResourceException("Resource not found", "/jrsdbaseclass.st", "jrsdbaseclass");
    }
    Reader in = new InputStreamReader(inis, "ISO-8859-1");
    StringTemplateGroup stg = new StringTemplateGroup(in, AngleBracketTemplateLexer.class);
    StringTemplate st = stg.getInstanceOf("baseclass");
    //package, baseclass, superclass, relpath
    st.setAttribute("package", md.getMappingPackage());
    st.setAttribute("baseclass", md.getMappingClassNameOnly());
    st.setAttribute("superclass", md.getSuperClass());
    st.setAttribute("relpath", relPath);
    return st.toString();
  }
  
  /**
   * Write the SQL for a table descriptor
   * @param td
   * @param content
   * @throws IOException
   */
  void writeSqlSource(TableDef td, String content)
  throws IOException
  {
    if (noSql) {
      lg.debug("Sql generation disabled, returning");
      return;
    }
    Date specDate = td.getJrsdSpec().getSpecDate();
    String prefix = "";
    prefix = makeSqlFilePrefix(td);
    String fileName = "";
    fileName = makeSqlFileName(td, prefix);
    File sqlFile = new File(sqlRoot, fileName);
    Date sqlFileDate = null;
    if (sqlFile.exists()) sqlFileDate = new Date(sqlFile.lastModified());
    if (overwrite || sqlFileDate == null || specDate.after(sqlFileDate)) {
      lg.info("Writing sql source file : "+sqlFile);
      FileOutputStream out = new FileOutputStream(sqlFile);
      out.write(content.getBytes(encoding));
      out.close();
    }
  }
    
  void emitXml(File outputFile, JrsdSpec spec)
  throws IOException
  {
    StringBuffer sb = new StringBuffer();
    XmlEmitter xem = new XmlEmitter();
    xem.produceXml(sb, spec);
    FileOutputStream fos = new FileOutputStream(outputFile);
    OutputStreamWriter osw = new OutputStreamWriter(fos, encoding);
    osw.write(sb.toString());
    osw.close();
    fos.close();
  }
  
  String[] listSqlFilesWithPrefix(String prefix)
  {
    FilenameFilter filter = new FilenameFilter() {
      /**
       * @see java.io.FilenameFilter#accept(java.io.File, java.lang.String)
       */
      public boolean accept(File dir, String name)
      {
        return name.startsWith(tempPrefix);
      }
    };
    tempPrefix = prefix;
    String[] names = sqlRoot.list(filter);
    return names;
  }

  /**
   * Detects if there is a conflicting SQL file. The use of -overwrite force
   * the overwrite of a conflicting file, else an exception is raised.
   * @param td
   * @param toDelete
   * @throws JrsdException
   */
  void detectSqlSourceConflicts(TableDef td, List<File> toDelete)
  throws JrsdException
  {
    Date specDate = td.getJrsdSpec().getSpecDate();
    String prefix = makeSqlFilePrefix(td);
    String regularFileName = makeSqlFileName(td, prefix);
    String[] fileNames = listSqlFilesWithPrefix(prefix);
    for (int i = 0; i < fileNames.length; i++) {
      File f = new File(sqlRoot, fileNames[i]);
      if (f.getName().equals(regularFileName)) continue;
      //file has same prefix, but it is not the regular file.
      //If the file is less thant one day old, just delete it (this is to ease
      //development), else throw an exception (some SQL was modified, but the
      //version number was not updated).
      long now = System.currentTimeMillis();
      Date lastModDate = new Date(f.lastModified());
      long age = now - f.lastModified();
      if (age < 1 * DAY) {
        if (overwrite || specDate.after(lastModDate)) {
          lg.info("Adding previous SQL source file to del list : '"+f+"'");
          toDelete.add(f);
        }
        else {
          lg.debug("Leaving SQL file untouched : "+f);
        }
      }
      else {
        String specFile = td.getJrsdSpec().getSpecFile();
        throw new JrsdException("There is a conflicting sql source file '" + f
            + "', change version of table (jrsd file " + specFile + ")");
      }
    }//for
  }
  
  private String makeSqlFileName(TableDef td, String prefix)
  {
    String fileName;
    fileName = prefix+td.getChecksum()+"."+fileext;
    return fileName;
  }

  private String makeSqlFilePrefix(TableDef td)
  {
    String prefix;
    prefix = "table-"+td.getName()+"_v"+td.getVersion()+"_";
    return prefix;
  }

  void makeDirs(File dir)
  throws IOException
  {
    if (!dir.exists()) {
      File parent = dir.getParentFile(); 
      if (parent == null) throw new IOException("No parent dir for '"+dir+"'");
      makeDirs(parent);
      boolean dirCreated = dir.mkdir();
      if (!dirCreated) {
        throw new IOException("Could not create directory '"+dir+"'");
      }
    }
  }
  
  public static boolean isEmpty(String str)
  {
    return str == null || str.trim().equals("");
  }
  
  public String getFileext() {
    return fileext;
  }
  public void setFileext(String fileext) {
    this.fileext = fileext;
  }

  public boolean isNoJava() {
    return noJava;
  }

  /**
   * 
   * @param noJava Control output of java files. If true, no java files will be
   *   generated. Default is false.
   */
  public void setNoJava(boolean noJava) {
    this.noJava = noJava;
  }

  public boolean isNoSql() {
    return noSql;
  }

  /**
   * 
   * @param noSql Control output of sql files. If true, no sql files will be
   *   generated. Default is false.
   */
  public void setNoSql(boolean noSql) {
    this.noSql = noSql;
  }

  public boolean isEmitXml() {
    return emitXml;
  }

  public void setEmitXml(boolean emitXml) {
    this.emitXml = emitXml;
  }
  
}
