package org.hmn.jrsd;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;

/*
 * 
 * Cree par Harry Karadimas a 16 sept. 2006
 */
/**
 * Jrsd task for ant. 
 * Use a nested fileset for input.<br>
 * Attributes :
 * <ul>
 *   <li>jrsddir : dir 
 *   <li>nojava : turn off java source generation
 *   <li>nosql : turn off sql generation
 *   <li>sqldir : dir where sql files will be generated. Mandatory except when
 *           nosql is true
 *   <li>srcdir : dir where java source files will be generated. Mandatory
 *           except when nojava is true
 *   <li>fileext : file extension for sql files (default is sql, java files
 *           always end with .java).
 *   <li>encoding : the encoding to use (default is platform encoding)
 * </ul>
 * @author karadimas
 */
public class JrsdTask
extends Task
{
  private Jrsd app;
  private ArrayList<FileSet> inputFileSets = new ArrayList<FileSet>();  
  private ArrayList<String> inputFiles = new ArrayList<String>();
  private String srcdir;  
  private String sqldir;
  private String xmldir;
  private String fileext = "sql";  
  private boolean booleanCharAltUsed;
  private boolean overwrite = false;
  private boolean debug = false;
  private boolean nojava = false;
  private boolean nosql = false;
  private boolean emitxml = false;
  private String encoding;
  
  /**
   * @return the encoding
   */
  public String getEncoding() {
    return encoding;
  }

  /**
   * @param encoding the encoding to set
   */
  public void setEncoding(String encoding) {
    this.encoding = encoding;
  }

  public JrsdTask() {
    app = new Jrsd();
  }
  
  /**
   * @see org.apache.tools.ant.Task#execute()
   */
  public void execute()
  throws BuildException 
  {
    if (nojava == false) verifyDirectory(srcdir, "Source root", "srcdir");    
    if (nosql == false) verifyDirectory(sqldir, "Sql root", "sqldir");
    app.srcRootPath = getSrcdirFile().getPath();
    app.sqlRootPath = getSqldirFile().getPath();    
    app.fileext = fileext;
    app.debug = debug;
    app.overwrite = overwrite;
    app.setEmitXml(emitxml);
    if (xmldir != null) app.xmlRootPath = xmldir;
    try {
      for (Iterator<FileSet> iter = inputFileSets.iterator(); iter.hasNext();) {
        FileSet fs = iter.next();
        DirectoryScanner ds = fs.getDirectoryScanner(getProject());
        String[] incFiles = ds.getIncludedFiles();
        File baseDir = ds.getBasedir();
        String baseDirPath = baseDir.getPath();
        for (int i = 0; i < incFiles.length; i++) {
          incFiles[i] = baseDirPath+File.separator+incFiles[i];
        }
        inputFiles.addAll(Arrays.asList(incFiles));
      }
      if (inputFiles.isEmpty()) {
        throw new JrsdException("No file to process");
      }
      app.inputs.addAll(inputFiles);      
      if (encoding != null) app.encoding = encoding;      
      app.init(); 
      app.run();
    }
    catch (JrsdException ex) {
      ex.printStackTrace();
      throw new BuildException(ex);
    }
  }
  
  public void addFileset(FileSet fs)
  {
    inputFileSets.add(fs);
  }
      
  private File verifyDirectory(String path, String desc, String opt)
  throws BuildException
  {
    if (isEmpty(path)) {
      throw new BuildException(desc+" path ("+opt+") missing");
    }
    File dir = getFile(path);
    if (!dir.exists()) {
      throw new BuildException(desc+" does not exist");
    }
    if (!dir.isDirectory()) {
      throw new BuildException(desc+" is not a directory");
    }
    return dir;
  }

  /**
   * Adjust the file relatively to the project's base dir 
   * (if the path is not absolute)
   * @param path
   * @return
   */
  public File getFile(String path)
  {
    File f = new File(path);
    if (f.isAbsolute()) return f;
    File baseDir = getProject().getBaseDir();
    return new File(baseDir, path);
  }

  public static boolean isEmpty(String str) {
    return str == null || str.trim().equals("");
  }
  
  public String getSrcdir() {
    return srcdir;
  }
  public File getSrcdirFile() {
    return getFile(srcdir);
  }
  public void setSrcdir(String d) {
    this.srcdir = d;
  }
  public String getSqldir() {
    return sqldir;
  }
  public File getSqldirFile() {
    return getFile(sqldir);
  }
  public void setSqldir(String sqldir) {
    this.sqldir = sqldir;
  }
  public String getFileext() {
    return fileext;
  }
  public void setFileext(String fileext) {
    this.fileext = fileext;
  }

  /**
   * @deprecated
   *   use {@link #isGenoracomments()}
   * @return
   */
  public boolean isGeneratecomments() {
    return app.generateComments;
  }

  /**
   * @deprecated
   *   use {@link #setGenoracomments(boolean)}
   * @param generatecomments
   */
  public void setGeneratecomments(boolean generatecomments) {
    app.generateComments = generatecomments;
  }
  
  public boolean isGenoracomments() {
    return app.generateComments;
  }
  
  public void setGenoracomments(boolean genoracomments) {
    app.generateComments = genoracomments;
  }
  
  public boolean isBooleancharaltused() {
    return booleanCharAltUsed;
  }

  public void setBooleancharaltused(boolean booleanCharAltUsed) {
    this.booleanCharAltUsed = booleanCharAltUsed;
  }
  public boolean isForcefieldsuc() {
    return app.forceFieldsToUppercase;
  }
  public void setForcefieldsuc(boolean b) {
    app.forceFieldsToUppercase = b;
  }
  public boolean isForcefieldslc() {
    return app.forceFieldsToLowercase;
  }
  public void setForcefieldslc(boolean b) {
    app.forceFieldsToLowercase = b;
  }

  public boolean isOverwrite() {
    return overwrite;
  }

  public void setOverwrite(boolean overwrite) {
    this.overwrite = overwrite;
  }

  public boolean isDebug() {
    return debug;
  }

  public void setDebug(boolean debug) {
    this.debug = debug;
  }

  public boolean isNojava() {
    return nojava;
  }

  public void setNojava(boolean nojava) {
    this.nojava = nojava;
  }

  public boolean isNosql() {
    return nosql;
  }

  public void setNosql(boolean nosql) {
    this.nosql = nosql;
  }

  public String getXmldir() {
    return xmldir;
  }

  public void setXmldir(String xmldir) {
    this.xmldir = xmldir;
  }

  public boolean isEmitxml() {
    return emitxml;
  }

  public void setEmitxml(boolean emitxml) {
    this.emitxml = emitxml;
  }
}
 