package org.hmn.jrsd;

import java.text.CharacterIterator;
import java.text.StringCharacterIterator;

import antlr.Token;

/*
 * 
 * Cree par Harry Karadimas a 30 juil. 2006
 */
/**
 * An SQL element, that can contain a comment and some literal sql.
 * @author karadimas
 */
public class SqlElementDef
{
  String comment;
  String literalSql;
  
  /** overridden in {@link ColumnDef} */
  public boolean isColumnDef() { return false; }
  
  public String getComment()
  {
    return comment;
  }
  
  public void setComment(String comment)
  {
    this.comment = comment;
  }
  
  public void setComment(Token t)
  {
    if (t == null) comment = null;
    else comment = t.getText();
  }
  
  /**
   * Get the comment, with apostrophes escaped, and new lines, start '*' and
   * initial spaces all replaced with a single space.
   * @return
   */
  public String getAeComment()
  {
    boolean lineStart = true;
    boolean firstSpace = true;
    if (comment == null) return null;
    StringBuffer sb = new StringBuffer();
    CharacterIterator iter = new StringCharacterIterator(comment);
    for (char c = iter.first(); c != CharacterIterator.DONE; c = iter.next()) {
      if (lineStart & (c==' '|c=='*'|c=='\t'|c=='\r'|c=='\n')) {
        if (firstSpace) { sb.append(' '); firstSpace = false; }        
      }
      else {
        lineStart = false;
        if (c == '\'') {
          sb.append(c); sb.append(c);  
        }
        else if (c == '\n' | c == '\r') {
          sb.append(' '); lineStart = true; firstSpace = true;
        }
        else sb.append(c);
      }
    }
    return sb.toString().trim();
  }
  
  /**
   * Remove first char from string. If string is empty (null or of length 0),
   *  return an empty string
   * ("")
   * @param s
   * @return A new string like s but with the first char removed
   */
  String remFirstChar(String s) {
    if (s == null || s.length() < 1) return "";
    return s.substring(1);
  }
  
  /**
   * Remove last char from string. If string is empty (null or of length 0),
   *  return an empty string
   * ("")
   * @param s
   * @return A new string like s but with the last char removed
   */
  String remLastChar(String s) {
    int len = s.length();
    if (s == null || len < 1) return "";
    return s.substring(0, len-1);
  }
  
  String stripLit(String s)
  {
    return remFirstChar(remLastChar(s));
  }
  
  /** make literal from token (calls .getText()) */
  String mkl(Token t)
  {
    if (t == null) return null;
    return t.getText();
  }
    
  /** make string from token (calls .getText()) */
  String mks(Token t)
  {
    if (t == null) return null;
    return t.getText();
  }
  
  public String getLiteralSql()
  {
    return literalSql;
  }
  public void setLiteralSql(String lsql)
  {
    this.literalSql = lsql;
  }
  public void setLiteralSql(Token t)
  {
    setLiteralSql(t == null ? null : t.getText());
  }
    
}
