package org.hmn.jrsd.rt;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import simpleorm.dataset.SFieldMeta;
import simpleorm.dataset.SFieldReference;
import simpleorm.dataset.SFieldScalar;

/*
 * 
 * Cree par Harry Karadimas a 10 aot 2006
 */
/**
 * A list of fields, which you can map to from a jrsd spec.
 * @author karadimas
 */
public class FieldList
{
  List<SFieldMeta> fields;
  
  /**
   * 
   */
  public FieldList(SFieldMeta[] fma)
  {
    fields = Arrays.asList(fma);
  }
  
  public String getColumnList()
  {
    StringBuffer sb = new StringBuffer();
    for (Iterator<SFieldMeta> iter = fields.iterator(); iter.hasNext();) {
      SFieldMeta f = (SFieldMeta) iter.next();
      String col = null;
      col = inferColumnName(f, ",");
      if (sb.length() > 0) sb.append(",");
      sb.append('"');
      sb.append(col);
      sb.append('"');
    }
    return sb.toString();
  }

  /**
   * Infer the column name(s)
   * @param f the meta of the field; must be a scalar field or a reference field.
   * @param sep
   * @return
   */
  private String inferColumnName(SFieldMeta f, String sep) {
    String col;
    if (f instanceof SFieldReference<?>) {
      //I discourage the usage of references with multiple keys, as the keys
      //are returned in a Set with no defined order
      SFieldReference<?> fr = (SFieldReference<?>) f;
      Set<SFieldScalar> fks = fr.getForeignKeyMetas();
      StringBuffer sb = new StringBuffer();
      for (SFieldScalar fk : fks) {
        if (sb.length() > 0) sb.append(sep);
        sb.append(fk.getColumnName());
      }
      col = fr.getFieldName();
    }
    else {
      SFieldScalar fs = (SFieldScalar) f;
      col = fs.getColumnName();
    }
    return col;
  }
  
  public String getQualifiedColumnList()
  {
    if (fields.isEmpty()) return "";
    SFieldMeta f = (SFieldMeta) fields.get(0);
    String table = f.getRecordMeta().getTableName();
    return getAliasedColumnList(table);
  }
  
  /**
   * Get a list of columns using the given alias. The names are quoted if 
   * necessary. 
   * <p>Ex 1 : <br>
   * Given a list of fields for <code>"NAME", "ADDRESS", "HAIR LENGTH"</code>, 
   * <code>getAliasedColumnList("a");</code> will return
   *  <code>' a.NAME, a.ADDRESS, a."HAIR LENGTH" '</code>
   * <p>Ex 2 : <br>
   * <code>getAliasedColumnList("MY ALIAS");</code> will return
   *  <code>' "MY ALIAS".NAME, "MY ALIAS".ADDRESS, "MY ALIAS"."HAIR LENGTH" '</code>
   * @see QColumn#quoteIfNecessary(String)
   * @param aliasNq The alias (non quoted). Quotes will be added only if
   *   necessary
   * @return A list of the columns in the list.
   */
  public String getAliasedColumnList(String aliasNq)
  {
    String alias = QColumn.quoteIfNecessary(aliasNq);
    StringBuffer sb = new StringBuffer();
    for (Iterator<SFieldMeta> iter = fields.iterator(); iter.hasNext();) {
      SFieldMeta f = iter.next();
      String colNq = inferColumnName(f, ",");
      if (sb.length() > 0) sb.append(",");
      sb.append(alias);
      sb.append('.');
      String col = QColumn.quoteIfNecessary(colNq);      
      sb.append(col);      
    }
    return sb.toString();    
  }
  
}
