package org.hmn.jrsd.rt.dbops;

import java.sql.SQLException;

import org.apache.log4j.Logger;
import org.hmn.jrsd.rt.PrimaryKeysRetrievable;
import org.hmn.jrsd.rt.ReflectionHelpers;

import simpleorm.sessionjdbc.SSessionJdbc;
import simpleorm.utils.SException;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;

/*
 * 
 * Cree par Harry Karadimas a 2 mai 08
 */
/**
 * Standard record insert operation. Returns 1 if the record was inserted
 * correctly. If the record already exists, throws an {@link SQLException}.
 * If exceptionRaisedIfExists is set to false, and the record already exists,
 * returns 0.
 */
public class DbRecordInsertOperation
extends DbUpdateOperation
{
  static Logger lg = Logger.getLogger(DbRecordInsertOperation.class);
  
  Object keys;
  SRecordMeta<?> meta;
  long sqy_bitSet;
  boolean exceptionRaisedIfExists = true;
  Object transportObject;
  
  public DbRecordInsertOperation(
      SRecordMeta<?> meta, Object keys, Object transportObject)
  {
    this(meta, keys, transportObject, 0L);
  }
  
  public DbRecordInsertOperation(
      SRecordMeta<?> meta, PrimaryKeysRetrievable transportObject)
  {
    this(meta, transportObject.getPrimaryKeyValues(), transportObject);
  }
  
  public DbRecordInsertOperation(
      SRecordMeta<?> meta, Object keys, Object transportObject, long sqy_bitSet)
  {
    this.meta = meta;    
    this.keys = keys;
    this.transportObject = transportObject;
    this.sqy_bitSet = sqy_bitSet;
  }
  
  public int execute()
  throws SQLException
  {
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    SRecordInstance rec = session.findOrCreate(meta, keys);
    if (!rec.isNewRow()) {
      if (exceptionRaisedIfExists) {
        throw new SQLException(
            "Record already exists "+meta.getTableName()+"#"+keys);
      }
      else return 0;
    }
    rec.setDirty(true);
    lg.info("rec : "+rec.getClass());
    try {
      ReflectionHelpers.callVoid("readFrom", rec, transportObject);
    }
    catch(Exception e) {
      throw new SException.Data(e);
    }
    return 1;
  }

  public boolean isExceptionRaisedIfExists() {
    return exceptionRaisedIfExists;
  }

  public void setExceptionRaisedIfExists(boolean exceptionRaisedIfExists) {
    this.exceptionRaisedIfExists = exceptionRaisedIfExists;
  }

}
