package org.hmn.jrsd.rt.dbops;

import java.sql.SQLException;

import org.hmn.jrsd.rt.PrimaryKeysRetrievable;
import org.hmn.jrsd.rt.ReflectionHelpers;

import simpleorm.sessionjdbc.SSessionJdbc;
import simpleorm.utils.SException;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;

/*
 * 
 * Cree par Harry Karadimas a 2 mai 08
 */
/**
 * Reads the given record, and copies the fields into the transport object.
 * If the record was not found using the given key(s), nothing is copied, and
 * the execution returns null. If the record was found, values have been
 * copied into the transport object, and the transport object itself is
 * returned.
 * If the transport object is null, the simpleorm record is returned instead.
 */
public class DbRecordReadOperation
extends DbObjectOperation
{
  SRecordMeta<?> meta;
  Object transportObject;
  Object keys;
  long sqy_bitSet;
  
  public DbRecordReadOperation(
      SRecordMeta<?> meta, Object keys, Object transportObject)
  {
    this(meta, transportObject, keys, 0L);
  }
  
  /**
   * Read record using the primary key values, and updates the fields of the
   * transport object using the fields just read.
   * @param meta
   * @param transportObject
   */
  public DbRecordReadOperation(
      SRecordMeta<?> meta, PrimaryKeysRetrievable transportObject)
  {
    this(meta, transportObject.getPrimaryKeyValues(), transportObject);
  }
  
  public DbRecordReadOperation(
      SRecordMeta<?> meta, Object keys, Object transportObject, long sqy_bitSet)
  {
    this.meta = meta;
    this.transportObject = transportObject;
    this.keys = keys;
    this.sqy_bitSet = sqy_bitSet;
  }
  
  public Object execute()
  throws SQLException
  {
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    SRecordInstance rec = session.findOrCreate(meta, keys);
    if (rec.isNewRow()) return null;
    if (transportObject == null) return rec;
    try {
      ReflectionHelpers.callVoid("writeTo", rec, transportObject);
      return transportObject;
    }
    catch(Exception e) {
      throw new SException.Data(e);
    }
  }

}
