package org.hmn.jrsd.rt.oracle;

/*
 * 
 * Cree par Harry Karadimas a 14 avr. 08
 */
/**
 * Utility to make "paginated" queries using ROWNUM and FIRST_ROWS 
 * optimizations.
 */
public class OraclePagination
{
  
  /**
   * Construct an SQL query, optimized for paginated output. The code was
   * inspired from a column in the "AskTom" website. The SQL query when
   * executed, returns only the rows that start at "startrow", and returns
   * at most "rowspp" rows per page. The SQL has the numbers inserted
   * literally, so that you can use your parameters as usual.
   * @param select The select clause (usually "select", but additonal
   *   text (usually hints) can be added.
   * @param colList The list of columns (ex : "NAME, BIRTHDATE, ID_NUMBER") that
   *   will be used by "select" and also the actual subselection of results.
   * @param rest The rest of the query (ex : "FROM PERSON ORDER BY NAME WHERE BIRTHDATE > ?")
   * @param startrow The number of the first row to return (starts with 1)
   * @param rowspp Number of rows per page
   * @return
   */
  public static String makeSql(
      String select, String colList, String rest, int startrow, int rowspp)
  {
    /* code trouv sur asktom */
    String sql = 
"select " + colList +
"  from "+
"  (select /*+ FIRST_ROWS("+rowspp+") */ a.*, ROWNUM rnum"+
"     from "+
"     (" + select + " " + colList + " " + rest + ") a"+
"     where ROWNUM <= "+(startrow + rowspp)+
"  )"+
"  where rnum  >= " + startrow;
    return sql;
  }
  
}
